/*
 * This program is a benchmark test for JDBC 2.0 drivers for Oracle database servers. 
 * This benchmark can be freely used, distributed and modified.
 *  i-net software 2002
 */

import java.util.*;
import java.io.*;
import java.sql.*;

public class Benchmark
{
    String driver = "com.inet.ora.OraDriver";
    String url = "jdbc:inetora:localhost:1521:orcl";
    String user = "system";
    String password = "manager";
    
    Connection con = null;
    
    public static void main( String [] args )
    {
        new Benchmark( args );
    }
    
    public Benchmark( String [] args )
    {
        // get commandline options
        getArgs( args );
        
        // print options
        System.out.println( "**********************************************************************" );
        System.out.println( "* i-net software JDBC 2.0 Benchmark" );
        System.out.println( "*" );
        System.out.println( "* Test started at " + new java.util.Date( System.currentTimeMillis() ) );
        System.out.println( "*" );
        System.out.println( "* Options:   -driver:   " + driver );
        System.out.println( "*            -url:      " + url );
        System.out.println( "*            -user:     " + user );
        System.out.println( "*            -password: " + password );
        System.out.println( "***********************************************************************\n" );
        
        // load the driver
        try {
            Class.forName( driver ).newInstance();
            System.out.println( "Driver successfully loaded...\n" );
        }
        catch( Throwable e ) {
            System.err.println( "Cannot find driver class <" + driver + ">." );
            System.err.println( "Please ensure that the driver is included in your classpath." );
            System.exit( 1 );
        }
        
        // logging and timeout
        try {
            DriverManager.setLoginTimeout( 10 );
            //DriverManager.setLogWriter( new PrintWriter( new FileOutputStream( "logging.txt" ), true ) );
        }
        catch( Throwable e ){}
        
        // get the connection
        try {
            con = DriverManager.getConnection( url, user, password );
            System.out.println( "Connection successfully established...\n" );
        }
        catch( SQLException e ) {
            System.err.println( "An error has occured while connecting the database:" );
            System.err.println( e.getMessage() );e.printStackTrace();
            System.exit( 1 );
        }
        
        // create the testtables
        createTestTables();
        
        // running the tests
        runMe();
        
        // drop the testtables
        dropTestTables();
    }
    
    public void getArgs( String [] args )
    {
        if( args.length % 2 != 0 ) {
            printUsage();
            System.exit( 1 );
        }
        
        for( int i = 0; i < args.length; i++ ) {
            if( args[i].equalsIgnoreCase( "-driver" ) )
                driver = args[++i];
            else if( args[i].equalsIgnoreCase( "-url" ) )
                url = args[++i];
            else if( args[i].equalsIgnoreCase( "-user" ) )
                user = args[++i];
            else if( args[i].equalsIgnoreCase( "-password" ) )
                password = args[++i];
            else {
                System.err.println( "...skipping unknown option <" + args[i] + ">" );
                i++;
            }
        }
    }
    
    public void printUsage()
    {
        System.out.println( "Usage: java -cp <%classpath%> Benchmark <options>\n" );
        System.out.println( "       Valid options are:\n" );
        System.out.println( "       -driver <driver>     (default com.inet.ora.OraDriver)" );
        System.out.println( "       -url <url>           (default jdbc:inetora:localhost:1521:orcl)" );
        System.out.println( "       -user <user>         (default system)" );
        System.out.println( "       -password <password> (default manager)" );
    }
    
    public void createTestTables()
    {
        Statement st = null;
        String q1 =
        
        "create table t_std( " + 
        "id number, " + 
        "fraction number(20,10), " + 
        "text varchar2(50), " + 
        "timestamp date, " +
        "shorttext char(50), " +
        "longtext long, " +
        "binary raw(50)" +
        " )";
        
        String q2 =
        
        "create table t_lob( " +
        "id number, " +
        "book clob, " +
        "image blob" +
        " )";
        
        // drop it first
        dropTestTables();
        
        // create the table
        try {
            st = con.createStatement();
            st.execute( q1 );
            st.execute( q2 );
            System.out.println( "Benchmark tables succesfully created...\n" );
        }
        catch( SQLException e ) {
            System.err.println( "Cannot create benchmark tables:" );
            System.err.println( e.getMessage() );
            System.exit( 1 );
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}   
        }
    }
    
    public void dropTestTables()
    {
        Statement st = null;
        
        try {
            st = con.createStatement();
            st.execute( "drop table t_std" );
            st.execute( "drop table t_lob" );
            System.out.println( "Benchmark tables succesfully dropped...\n" );
        }
        catch( SQLException e ) {
            // do nothing
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}   
        }
    }
    
    public void runMe()
    {
        try {
            // print driver name and version
            System.out.println( "Running tests...\n" );
            System.out.println( "Driver:         " + con.getMetaData().getDriverName() );
            System.out.println( "Driver Version: " + con.getMetaData().getDriverVersion() + "\n" );
            
            // run the tests
            classicStdInsert();
            getAll();
            preparedInsert();
            getNumbers();
            getReals();
            getDates();
            getText();
            getBinaries();
            getNumbersFromScroll();
            getRealsFromScroll();
            getDatesFromScroll();
            getTextFromScroll();
            getBinariesFromScroll();
            lobInsert();
            getBlobs();
            getClobs();
            getBlobsFromScroll();
            getClobsFromScroll();
            
            System.out.println( "\n*** Test finnished at " + new java.util.Date( System.currentTimeMillis() ) + "***\n" );
        }
        catch( SQLException e ) {
            System.err.println( "An unexpected error occurs:" );
            System.err.println( e.getMessage() );
            System.exit( 1 );
        }
    }
    
    /* insert 100000 reccords with standard insert statement */
    public boolean classicStdInsert()
    {
        Statement st = null;
        long time;
        
        tabln( "classicStdInsert()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            for( int i = 0; i < 100000; i++ )
                st.executeUpdate( "insert into t_std( id, fraction, text, timestamp, shorttext, longtext, binary ) values( 1, 0123456789.0123456789, 'abcdefghijklmnopqrstuvwxyz', to_date( '01-12-2002', 'dd-mm-yyyy' ), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', '01234567890abcdefghijklmnopqrstuvwxyz', '00112233445566778899AABBCCDDEEFF' )" );
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time )  );
        return true;
    }
    
    /* insert 100000 records with a prepared statement */
    public boolean preparedInsert()
    {
        PreparedStatement ps = null;
        long time;
        
        tabln( "preparedInsert()..." );
        
        try {
            warmUp();
            ps = con.prepareStatement( "truncate table t_std" );
            ps.execute();
            ps.close();
            ps = con.prepareStatement( "insert into t_std( id, fraction, text, timestamp, shorttext, longtext, binary ) values( ?, ?, ?, ?, ?, ?, ? )" );
            
            ps.setDouble( 2, 0123456789.0123456789 );
            ps.setString( 3, "abcdefghijklmnopqrstuvwxyz" );
            ps.setDate( 4, new java.sql.Date( System.currentTimeMillis() ) );
            ps.setString( 5, "ABCDEFGHIJKLMNOPQRSTUVWXYZ" );
            ps.setString( 6, "01234567890abcdefghijklmnopqrstuvwxyz" );
            ps.setBytes( 7, new byte[16] );
            
            time = System.currentTimeMillis();
            
            for( int i = 0; i < 100000; i++ ) {
                ps.setInt( 1, i );
                ps.executeUpdate();
            }
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( ps != null )ps.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }

    /* insert 10 reccords with lobs */
    public boolean lobInsert()
    {
        PreparedStatement ps = null;
        long time;
        
        tabln( "lobInsert()..." );
        
        try {
            warmUp();
            ps = con.prepareStatement( "insert into t_lob( id, book, image ) values( ?, ?, ? )" );
            
            byte [] b = new byte[65536]; 
            Arrays.fill( b, (byte)'A' );
            StringClob cl = new StringClob( new String( b ) );
            ByteArrayBlob bl = new ByteArrayBlob( b );
            
            time = System.currentTimeMillis();
            
            for( int i = 0; i < 10; i++ ) {
                ps.setInt( 1, i );
                ps.setClob( 2, cl );
                ps.setBlob( 3, bl );
                ps.executeUpdate();
            }
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( ps != null )ps.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get all the values from the testtable */
    public boolean getAll()
    {
        Statement st = null;
        long time;
        
        tabln( "getAll()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select * from t_std" );
            rs.setFetchSize( 256 );
            ResultSetMetaData rsmd = rs.getMetaData();
             
            while( rs.next() )
                for( int i = 1; i <= rsmd.getColumnCount(); i++ )
                    rs.getObject( i );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get numbers */
    public boolean getNumbers()
    {
        Statement st = null;
        long time;
        
        tabln( "getNumbers()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select id from t_std" );
            rs.setFetchSize( 256 );
             
            while( rs.next() )
                rs.getInt( "id" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get numbers with fraction */
    public boolean getReals()
    {
        Statement st = null;
        long time;
        
        tabln( "getReals()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select fraction from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getDouble( "fraction" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get dates */
    public boolean getDates()
    {
        Statement st = null;
        long time;
        
        tabln( "getDates()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select timestamp from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getDate( "timestamp" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get characters */
    public boolean getText()
    {
        Statement st = null;
        long time;
        
        tabln( "getText()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select text from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getString( "text" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get clob */
    public boolean getClobs()
    {
        Statement st = null;
        long time;
        
        tabln( "getClobs()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select book from t_lob" );
            rs.setFetchSize( 256 );
            
            while( rs.next() ) {
                Clob cl = rs.getClob( "book" );
                InputStream in = cl.getAsciiStream();
            }
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get blobs */
    public boolean getBlobs()
    {
        Statement st = null;
        long time;
        
        tabln( "getBlobs()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select image from t_lob" );
            rs.setFetchSize( 256 );
            
            while( rs.next() ) {
                Blob bl = rs.getBlob( "image" );
                InputStream in = bl.getBinaryStream();
            }
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    /* get binary data */
    public boolean getBinaries()
    {
        Statement st = null;
        long time;
        
        tabln( "getBinaries()..." );
        
        try {
            warmUp();
            st = con.createStatement();
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select binary from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getBytes( "binary" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    // the same tests with scrollable resultsets
    
    public boolean getNumbersFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getNumbersFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select id from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getInt( "id" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public boolean getRealsFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getRealsFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select fraction from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getDouble( "fraction" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public boolean getDatesFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getDatesFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select timestamp from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getDate( "timestamp" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public boolean getTextFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getTextFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select text from t_std" );
            rs.setFetchSize( 256 );
             
            while( rs.next() )
                rs.getString( "text" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public boolean getClobsFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getClobsFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select book from t_lob" );
            rs.setFetchSize( 256 );
            
            while( rs.next() ) {
                Clob cl = rs.getClob( "book" );
                InputStream in = cl.getAsciiStream();
            }
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public boolean getBlobsFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getBlobsFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select image from t_lob" );
            rs.setFetchSize( 256 );
            
            while( rs.next() ) {
                Blob bl = rs.getBlob( "image" );
                InputStream in = bl.getBinaryStream();
            }
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public boolean getBinariesFromScroll()
    {
        Statement st = null;
        long time;
        
        tabln( "getBinariesFromScroll()..." );
        
        try {
            warmUp();
            st = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE );
            time = System.currentTimeMillis();
            
            ResultSet rs = st.executeQuery( "select binary from t_std" );
            rs.setFetchSize( 256 );
            
            while( rs.next() )
                rs.getBytes( "binary" );
             
            rs.close();    
                
            time = System.currentTimeMillis() - time;
        }
        catch( Throwable e ) {
            System.out.println( "failed, " + e.toString() );
            return false;
        }
        finally {
            try{ if( st != null )st.close(); }catch( SQLException e ){}  
        }
        
        System.out.println( "ok, required time: " + getTime( time ) );
        return true;
    }
    
    public void warmUp()
    {
        try {
            Statement st = con.createStatement();
            ResultSet rs = st.executeQuery( "select user from dual" );
            while( rs.next() )
                rs.getObject( 1 );
            rs.close();
            st.close();
        }
        catch( SQLException e ) {
            // do nothing
        }
    }
    
    public String getTime( long millis )
    {
        String s = "";
        long time;
        
        time = millis / 3600000;
        if( time != 0 ) {
            s += time + "h";
            millis -= (time * 3600000);
        }
        time = millis / 60000;
        if( time != 0 ) {
            s += time + "m";
            millis -= (time * 60000);
        }
        time = millis / 1000;
        if( time != 0 ) {
            s += time + "s";
            millis -= (time * 1000);
        }
        s += millis + "ms";
        
        return s;
    }
    
    public void tabln( String s )
    {
        StringBuffer buf = new StringBuffer( "                              " );
        buf.insert( 0, s );
        buf.setLength( 30 );
        System.out.print( buf.toString() );
    }
}
